@@ -78,15 +78,11 @@ class Server(object):
     :param auth: If True, enable authentication with default mechanisms. May
                  also be given as a list of SASL mechanism names to support,
                  e.g. ``['PLAIN', 'LOGIN', 'CRAM-MD5']``.
-    :param tls: Optional dictionary of TLS settings passed directly as
-                keyword arguments to :class:`gevent.ssl.SSLSocket`.
+    :param context: Enables SSL encryption on connected sockets using the
+                    information given in the context.
     :param tls_immediately: If True, the socket will be encrypted
-                            immediately.
-    :param tls_wrapper: Optional function that takes a socket and the ``tls``
-                        dictionary, creates a new encrypted socket, performs
-                        the TLS handshake, and returns it. The default uses
-                        :class:`~gevent.ssl.SSLSocket`.
-    :type tls_immediately: True or False
+                            immediately on connection rather than advertising
+                            ``STARTTLS``.
     :param command_timeout: Optional timeout waiting for a command to be
                             sent from the client.
     :param data_timeout: Optional timeout waiting for data to be sent from
@@ -95,12 +91,12 @@ class Server(object):
     """
 
     def __init__(self, socket, handlers, auth=False,
-                 tls=None, tls_immediately=False, tls_wrapper=None,
+                 context=None, tls_immediately=False,
                  command_timeout=None, data_timeout=None):
         self.handlers = handlers
         self.extensions = Extensions()
 
-        self.io = IO(socket, tls_wrapper)
+        self.io = IO(socket)
 
         self.bannered = False
         self.have_mailfrom = None
@@ -108,11 +104,14 @@ class Server(object):
         self.ehlo_as = None
         self.authed = False
 
+        self.context = context
+        self.tls_immediately = tls_immediately
+
         self.extensions.add('8BITMIME')
         self.extensions.add('PIPELINING')
         self.extensions.add('ENHANCEDSTATUSCODES')
         self.extensions.add('SMTPUTF8')
-        if tls and not tls_immediately:
+        if self.context and not tls_immediately:
             self.extensions.add('STARTTLS')
         if auth:
             if isinstance(auth, list):
@@ -122,13 +121,6 @@ class Server(object):
             auth_session = AuthSession(auth_obj, self.io)
             self.extensions.add('AUTH', auth_session)
 
-        if tls:
-            self.tls = tls.copy()
-            self.tls.setdefault('server_side', True)
-        else:
-            self.tls = None
-        self.tls_immediately = tls_immediately
-
         self.command_timeout = command_timeout
         self.data_timeout = data_timeout or command_timeout
 
@@ -165,7 +157,7 @@ class Server(object):
         self.have_rcptto = None
 
     def _encrypt_session(self):
-        if not self.io.encrypt_socket(self.tls):
+        if not self.io.encrypt_socket_server(self.context):
             return False
         self._call_custom_handler('TLSHANDSHAKE')
         return True
@@ -193,7 +185,7 @@ class Server(object):
         :raises: :class:`~slimta.smtp.ConnectionLost` or unhandled exceptions.
 
         """
-        if self.tls and self.tls_immediately:
+        if self.context and self.tls_immediately:
             if not self._encrypt_session():
                 tls_failure.send(self.io, flush=True)
                 return
@@ -259,8 +251,12 @@ class Server(object):
         if not self.bannered:
             bad_sequence.send(self.io)
             return
+        elif not ehlo_as:
+            bad_arguments.send(self.io)
+            return
 
-        reply = Reply('250', 'Hello '+ehlo_as.decode('ascii'))
+        ehlo_as = ehlo_as.decode('utf-8')
+        reply = Reply('250', 'Hello '+ehlo_as)
         reply.enhanced_status_code = False
         self._call_custom_handler('EHLO', reply, ehlo_as)
 
@@ -279,10 +275,15 @@ class Server(object):
         if not self.bannered:
             bad_sequence.send(self.io)
             return
+        elif not ehlo_as:
+            bad_arguments.send(self.io)
+            return
 
-        reply = Reply('250', 'Hello '+ehlo_as.decode('ascii'))
+        ehlo_as = ehlo_as.decode('utf-8')
+        reply = Reply('250', 'Hello '+ehlo_as)
         reply.enhanced_status_code = False
         self._call_custom_handler('HELO', reply, ehlo_as)
+
         reply.send(self.io)
         self._check_close_code(reply)
 
