@@ -1,7 +1,5 @@
 
-import unittest
-
-from assertions import *
+import unittest2 as unittest
 
 from slimta.policy import QueuePolicy, RelayPolicy
 
@@ -10,11 +8,11 @@ class TestPolicy(unittest.TestCase):
 
     def test_queuepolicy_interface(self):
         qp = QueuePolicy()
-        assert_raises(NotImplementedError, qp.apply, None)
+        self.assertRaises(NotImplementedError, qp.apply, None)
 
     def test_relaypolicy_interface(self):
         rp = RelayPolicy()
-        assert_raises(NotImplementedError, rp.apply, None)
+        self.assertRaises(NotImplementedError, rp.apply, None)
 
 
 # vim:et:fdm=marker:sts=4:sw=4:ts=4
