@@ -1,7 +1,5 @@
 
-import unittest
-
-from assertions import *
+import unittest2 as unittest
 
 from slimta.relay.blackhole import BlackholeRelay
 from slimta.envelope import Envelope
@@ -14,7 +12,7 @@ class TestBlackholeRelay(unittest.TestCase):
         env = Envelope()
         blackhole = BlackholeRelay()
         ret = blackhole.attempt(env, 0)
-        assert_equal('250', ret.code)
+        self.assertEqual('250', ret.code)
 
     def test_attempt_policies(self):
         class BadPolicy(RelayPolicy):
@@ -24,7 +22,7 @@ class TestBlackholeRelay(unittest.TestCase):
         blackhole = BlackholeRelay()
         blackhole.add_policy(BadPolicy())
         ret = blackhole._attempt(env, 0)
-        assert_equal('250', ret.code)
+        self.assertEqual('250', ret.code)
 
 
 # vim:et:fdm=marker:sts=4:sw=4:ts=4
