@@ -1,6 +1,6 @@
 import unittest2 as unittest
 from mox3.mox import MoxTestBase, IsA
-from gevent.ssl import SSLSocket, SSLError
+from gevent.ssl import SSLSocket, SSLContext, SSLError
 from pysasl import SASLAuth
 
 from slimta.smtp.server import Server
@@ -14,16 +14,16 @@ class TestSmtpServer(unittest.TestCase, MoxTestBase):
         super(TestSmtpServer, self).setUp()
         self.sock = self.mox.CreateMock(SSLSocket)
         self.sock.fileno = lambda: -1
-        self.tls_args = {'certfile': '/dev/null', 'server_side': True}
+        self.sock.getpeername = lambda: ('test', 0)
+        self.context = self.mox.CreateMock(SSLContext)
+        self.context.session_stats = lambda: {}
 
     def test_starttls_extension(self):
-        s = Server(None, None, tls=False)
+        s = Server(None, None)
         self.assertFalse('STARTTLS' in s.extensions)
-        s = Server(None, None, tls={}, tls_immediately=False)
-        self.assertFalse('STARTTLS' in s.extensions)
-        s = Server(None, None, tls=self.tls_args, tls_immediately=False)
+        s = Server(None, None, context=self.context)
         self.assertTrue('STARTTLS' in s.extensions)
-        s = Server(None, None, tls=self.tls_args, tls_immediately=True)
+        s = Server(None, None, context=self.context, tls_immediately=True)
         self.assertFalse('STARTTLS' in s.extensions)
 
     def test_recv_command(self):
@@ -84,25 +84,19 @@ class TestSmtpServer(unittest.TestCase, MoxTestBase):
         s.handle()
 
     def test_tls_immediately(self):
-        sock = self.mox.CreateMockAnything()
-        sock.fileno = lambda: -1
-        sock.tls_wrapper(sock, self.tls_args).AndReturn(sock)
-        sock.sendall(b'220 ESMTP server\r\n')
-        sock.recv(IsA(int)).AndReturn(b'QUIT\r\n')
-        sock.sendall(b'221 2.0.0 Bye\r\n')
+        self.context.wrap_socket(self.sock, server_side=True).AndReturn(self.sock)
+        self.sock.sendall(b'220 ESMTP server\r\n')
+        self.sock.recv(IsA(int)).AndReturn(b'QUIT\r\n')
+        self.sock.sendall(b'221 2.0.0 Bye\r\n')
         self.mox.ReplayAll()
-        s = Server(sock, None, tls=self.tls_args, tls_immediately=True,
-                   tls_wrapper=sock.tls_wrapper)
+        s = Server(self.sock, None, context=self.context, tls_immediately=True)
         s.handle()
 
     def test_tls_immediately_sslerror(self):
-        sock = self.mox.CreateMockAnything()
-        sock.fileno = lambda: -1
-        sock.tls_wrapper(sock, self.tls_args).AndRaise(SSLError())
-        sock.sendall(b'421 4.7.0 TLS negotiation failed\r\n')
+        self.context.wrap_socket(self.sock, server_side=True).AndRaise(SSLError())
+        self.sock.sendall(b'421 4.7.0 TLS negotiation failed\r\n')
         self.mox.ReplayAll()
-        s = Server(sock, None, tls=self.tls_args, tls_immediately=True,
-                   tls_wrapper=sock.tls_wrapper)
+        s = Server(self.sock, None, context=self.context, tls_immediately=True)
         s.handle()
 
     def test_ehlo(self):
@@ -180,39 +174,35 @@ class TestSmtpServer(unittest.TestCase, MoxTestBase):
         self.assertEqual('there', s.ehlo_as)
 
     def test_starttls(self):
-        sock = self.mox.CreateMockAnything()
-        sock.fileno = lambda: -1
-        sock.sendall(b'220 ESMTP server\r\n')
-        sock.recv(IsA(int)).AndReturn(b'EHLO there\r\n')
-        sock.sendall(b'250-Hello there\r\n250 STARTTLS\r\n')
-        sock.recv(IsA(int)).AndReturn(b'STARTTLS\r\n')
-        sock.sendall(b'220 2.7.0 Go ahead\r\n')
-        sock.tls_wrapper(sock, self.tls_args).AndReturn(sock)
-        sock.recv(IsA(int)).AndReturn(b'QUIT\r\n')
-        sock.sendall(b'221 2.0.0 Bye\r\n')
+        self.sock.sendall(b'220 ESMTP server\r\n')
+        self.sock.recv(IsA(int)).AndReturn(b'EHLO there\r\n')
+        self.sock.sendall(b'250-Hello there\r\n250 STARTTLS\r\n')
+        self.sock.recv(IsA(int)).AndReturn(b'STARTTLS\r\n')
+        self.sock.sendall(b'220 2.7.0 Go ahead\r\n')
+        self.context.wrap_socket(self.sock, server_side=True).AndReturn(self.sock)
+        self.sock.recv(IsA(int)).AndReturn(b'QUIT\r\n')
+        self.sock.sendall(b'221 2.0.0 Bye\r\n')
         self.mox.ReplayAll()
-        s = Server(sock, None, tls=self.tls_args, tls_wrapper=sock.tls_wrapper)
+        s = Server(self.sock, None, context=self.context)
         s.extensions.reset()
         s.extensions.add('STARTTLS')
         s.handle()
         self.assertEqual(None, s.ehlo_as)
 
     def test_starttls_bad(self):
-        sock = self.mox.CreateMockAnything()
-        sock.fileno = lambda: -1
-        sock.sendall(b'220 ESMTP server\r\n')
-        sock.recv(IsA(int)).AndReturn(b'STARTTLS\r\n')
-        sock.sendall(b'503 5.5.1 Bad sequence of commands\r\n')
-        sock.recv(IsA(int)).AndReturn(b'STARTTLS badarg\r\n')
-        sock.sendall(b'501 5.5.4 Syntax error in parameters or arguments\r\n')
-        sock.recv(IsA(int)).AndReturn(b'EHLO there\r\n')
-        sock.sendall(b'250-Hello there\r\n250 STARTTLS\r\n')
-        sock.recv(IsA(int)).AndReturn(b'STARTTLS\r\n')
-        sock.sendall(b'220 2.7.0 Go ahead\r\n')
-        sock.tls_wrapper(sock, self.tls_args).AndRaise(SSLError())
-        sock.sendall(b'421 4.7.0 TLS negotiation failed\r\n')
+        self.sock.sendall(b'220 ESMTP server\r\n')
+        self.sock.recv(IsA(int)).AndReturn(b'STARTTLS\r\n')
+        self.sock.sendall(b'503 5.5.1 Bad sequence of commands\r\n')
+        self.sock.recv(IsA(int)).AndReturn(b'STARTTLS badarg\r\n')
+        self.sock.sendall(b'501 5.5.4 Syntax error in parameters or arguments\r\n')
+        self.sock.recv(IsA(int)).AndReturn(b'EHLO there\r\n')
+        self.sock.sendall(b'250-Hello there\r\n250 STARTTLS\r\n')
+        self.sock.recv(IsA(int)).AndReturn(b'STARTTLS\r\n')
+        self.sock.sendall(b'220 2.7.0 Go ahead\r\n')
+        self.context.wrap_socket(self.sock, server_side=True).AndRaise(SSLError())
+        self.sock.sendall(b'421 4.7.0 TLS negotiation failed\r\n')
         self.mox.ReplayAll()
-        s = Server(sock, None, tls=self.tls_args, tls_wrapper=sock.tls_wrapper)
+        s = Server(self.sock, None, context=self.context)
         s.extensions.reset()
         s.extensions.add('STARTTLS')
         s.handle()
@@ -437,7 +427,7 @@ class TestSmtpServer(unittest.TestCase, MoxTestBase):
         self.sock.recv(IsA(int)).AndReturn(b'QUIT\r\n')
         self.sock.sendall(b'221 2.0.0 Bye\r\n')
         self.mox.ReplayAll()
-        s = Server(self.sock, None, tls=False)
+        s = Server(self.sock, None)
         s.handle()
 
     def test_gather_params(self):
